import java.awt.Dimension;

import javax.media.Buffer;
import javax.media.Effect;
import javax.media.Format;
import javax.media.ResourceUnavailableException;
import javax.media.format.RGBFormat;

/**
 * @author Horst Eidenberger (hme@ims.tuwien.ac.at)
 * adaptation: Heinrich Hussmann, Bettina Conradi, Sara Streng
 * @version 1
 */
public class GrayEffect implements Effect {
	/** Supported input formats */
	private Format[] inputFormats;
	/** Supported output formats */
	private Format[] outputFormats;

	/**
	 * This is where the actual effect is implemented. The method
	 * runs through all pixels (read out of the in buffer), 
	 * manipulates them and puts them back in the out buffer.
	 * @param in in buffer
	 * @param out out buffer
	 * @return status
	 */
	public synchronized int process(Buffer in, Buffer out) {
		
		// copy input buffer to output buffer
		out.copy(in);
		int[] data = (int[]) out.getData();

		// get height and width of current frame
		RGBFormat inFormat = (RGBFormat) in.getFormat();
		Dimension inSize = inFormat.getSize();
		int width = (int) inSize.getWidth();
		int height = (int) inSize.getHeight();
		
		// pixel coordinates
		int x, y;
		// position of the pixel in the data[]
		int offBuffer;
		
		// run through all pixels
		for(y=0; y<height; y++) {
			 for(x=0; x<width; x++) {
				 
				 // calculate position of the pixel in the data[]
				 offBuffer = y*width + x;
				 
				 // get RGB value of current pixel
			     int rgb = data[offBuffer];
			     
			     // extract values for red, green and blue from RGB value
			     // by doing a bit shift
			     int red 	= (rgb >> 16) &0xff;
			     int green	= (rgb >> 8) &0xff;
			     int blue	= rgb &0xff;
			     
			     // formula that converts color to gray:
			     int luminance = (int) Math.round(0.3*red + 0.59*green + 0.11*blue);
			     
			     // put values for red, green and blue back into one RGB value
			     int gray = luminance << 16| luminance << 8 | luminance;
			             	
			     // overwrite data
			     data[offBuffer] = gray;
			  }                                    
		}
		
		// return the information that the processing was successful
		return BUFFER_PROCESSED_OK;
	}
			       
	// not needed for this simple codec
	public void open() throws ResourceUnavailableException {
	}

	// not needed for this simple codec
	public void close() {
	}

	// not needed for this simple codec
	public void reset() {
	}

	// not needed for this simple codec
	public synchronized Object getControl(String controlType) {
		return null;
	}

	// not needed for this simple codec
	public synchronized Object[] getControls() {
		return new Object[0];
	}

	/**
	 * Constructor, setting input and output formats
	 */
	public GrayEffect() {
		inputFormats = new Format[] { new RGBFormat(null, Format.NOT_SPECIFIED,
				Format.intArray, Format.NOT_SPECIFIED, 32,
				Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };

		outputFormats = new Format[] { new RGBFormat(null,
				Format.NOT_SPECIFIED, Format.intArray, Format.NOT_SPECIFIED,
				32, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };
	}

	/**
	 * Return list of supported input format
	 * @return supported list of input format
	 */
	public Format[] getSupportedInputFormats() {
		return inputFormats;
	}

	/**
	 * Return list of supported output format
	 * @param input input format
	 * @return supported list of output format for a given input format
	 */
	public Format[] getSupportedOutputFormats(Format input) {
		if (input == null)
			return (outputFormats);
		if (input instanceof RGBFormat)
			return (new Format[] { input, outputFormats[0] });
		else
			return (outputFormats);
	}

	/**
	 * Set input format (if RGBFormat) or returns the default input format
	 * @param input input format
	 * @return input format
	 */
	public Format setInputFormat(Format input) {
		// declare return value
		Format inputFormat;
		
		if (input instanceof RGBFormat)
			inputFormat = input;
		else
			inputFormat = inputFormats[0];
		return (inputFormat);
	}

	/**
	 * Set output format (if RGBFormat) or returns the default output format
	 * @param output output format
	 * @return output format
	 */
	public Format setOutputFormat(Format output) {
		// declare return value
		Format outputFormat;
		
		if (output instanceof RGBFormat)
			outputFormat = output;
		else
			outputFormat = outputFormats[0];
		return (outputFormat);
	}

	/**
	 * Return name that describes the effect
	 * @return name of effect
	 */
	public String getName() {
		return "gray effect";
	}	
}
